/*	$Id: lock.c,v 1.1 1999/12/08 03:50:47 leonard Exp $	*/
/* Copyright 1999, David Leonard. All rights reserved.
 *
 * Based on sys-bsd.c from pppd, with the following copyright:
 *
 * Copyright (c) 1989 Carnegie Mellon University.
 * Copyright (c) 1995 The Australian National University.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms are permitted
 * provided that the above copyright notice and this paragraph are
 * duplicated in all such forms and that any documentation,
 * advertising materials, and other materials related to such
 * distribution and use acknowledge that the software was developed
 * by Carnegie Mellon University and The Australian National University.
 * The names of the Universities may not be used to endorse or promote
 * products derived from this software without specific prior written
 * permission.
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
 * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 */

#include <stdio.h>
#include <string.h>
#include <err.h>
#include <fcntl.h>
#include <unistd.h>
#include <stdlib.h>
#include <signal.h>
#include <errno.h>

#include "vox.h"
#include "run.h"
#include "lock.h"

int locked;
char *lockname = NULL;

#define LOCK_PREFIX     "/var/spool/lock/LCK.."

void
lock(onoff)
        int onoff;
{
	char *base;
	char lockbuf[12];
	int fd;
	int n;
	pid_t pid;
	int warned = 0;

	if (locked == onoff) {
		if (verbose)
			fprintf(stderr, "already %slocked\n",
				locked ? "" : "un");
		return;
	}

	if (lockname == NULL) {
		/* compute the name of the lock file */
		if ((base = strrchr(modemdev, '/')) != NULL)
			base++;
		else
			base = modemdev;
		lockname = malloc(strlen(base) + strlen(LOCK_PREFIX) + 1);
		if (lockname == NULL)
			err(1, "malloc");
		sprintf(lockname, "%s%s", LOCK_PREFIX, base);
	}

	if (onoff == 0 && locked) {
		if (verbose)
			fprintf(stderr, "unlocked %s\n", lockname);
		unlink(lockname);
		locked = 0;
		return;
	}

	while ((fd = open(lockname, O_EXCL | O_CREAT | O_CREAT | O_RDWR))==-1) {
		if (errno == EEXIST && (fd = open(lockname, O_RDONLY, 0))!=-1) {
			/* Read the lock file to see who has it */
			n = read(fd, lockbuf, sizeof lockbuf - 1);
			if (n <= 0)
				err(1, "%s", lockname);
			close(fd);
			lockbuf[n] = '\0';
			pid = atoi(lockbuf);
			if (kill(pid, 0) == -1 && errno == ESRCH) {
				if (unlink(lockname) == -1)
					warn("Couldn't remove stale lock %s",
					    lockname);
				else {
					warn("Removed stale lock %s, pid %d",
					    lockname, pid);
					continue;
				}
			} else {
				if (!warned)
					warn("Device %s is locked by pid %d",
					    lockname, pid);
				warned = 1;
			}
		} else
			err(1, "Can't create lock file %s", lockname);
		sleep(1);
	}

	if (verbose)
		fprintf(stderr, "locking file %s\n", lockname);
	snprintf(lockbuf, sizeof lockbuf, "%10d\n", getpid());
	write(fd, lockbuf, strlen(lockbuf));
	close(fd);
	locked = 1;
}
