/*	$Id: main.c,v 1.4 1999/12/08 03:48:01 leonard Exp $	*/
/* Copyright 1999, David Leonard. All rights reserved. */

#include <stdio.h>
#include <fcntl.h>
#include <stdlib.h>
#include <termios.h>
#include <signal.h>
#include <err.h>
#include "vox.h"
#include "lock.h"
#include "run.h"

char *modemdev;

void
usage(progname)
	char *progname;
{

	fprintf(stderr, "usage: %s [-l modem] [-s speed] [-v] script\n", 
	    progname);
}

/* During cleanup, unlock */
static void
unlock()
{
	lock(0);
}

/* Catch a signal and die. */
static void
die(sig)
	int sig;
{
	exit(1);
}

int
main(argc, argv)
	int argc;
	char *argv[];
{
	extern FILE *yyin;
	int baud = 0;
	struct termios t;
	char *scriptfile;
	char *progname;
	extern int optind;
	extern char *optarg;
	char ch;

	modemdev = NULL;
	progname = argv[0];
	while ((ch = getopt(argc, argv, "l:s:v")) != -1) {
		switch (ch) {
		case 'v':
			verbose = 1;
			break;
		case 's':
			baud = atoi(optarg);
			break;
		case 'l':
			modemdev = optarg;
			break;
		case '?':
		default:
			usage(progname);
			exit(1);
		}
	}
	argc -= optind;
	argv += optind;

	if (argc != 1) {
		usage(progname);
		exit(1);
	}

	scriptfile = *argv;
	
	/* Open the script */
	yyin = fopen(scriptfile, "r");
	if (yyin == NULL)
		err(1, "%s", scriptfile);

	/* Parse the script into a command list */
	srcname = scriptfile;
	srcline = 0;
	initlabels();
	yyparse();
	fclose(yyin);
	fix_dests(first);
	check_closure(first);

	/* XXX use ttyname(0) in the future? */
	if (modemdev == NULL)
		modemdev = "/dev/modem";

	/* Open the modem */
	modemfd = open(modemdev, O_RDWR | O_NONBLOCK, 0666);
	if (modemfd == -1)
		err(1, "%s", modemdev);

	/* Set the tty to raw mode. */
	if (tcgetattr(modemfd, &t) == -1)
		warn("%s: tcgetattr", modemdev); 
	else {
		cfmakeraw(&t);
		if (baud != 0 && cfsetspeed(&t, baud) == -1)
			err(1, "%s: cfsetspeed", modemdev);
		if (tcsetattr(modemfd, TCSAFLUSH, &t) == -1)
			warn("%s: tcsetattr", modemdev);
	}

	/* On common termination signals, shut down. */
	signal(SIGINT, die);
	signal(SIGTERM, die);

	atexit(unlock);

	/* Run the script */
	run(first);

	exit(0);
}
