/*	$Id: vox.c,v 1.3 1999/12/08 03:49:15 leonard Exp $	*/
/* Copyright 1999, David Leonard. All rights reserved. */

#include <stdio.h>
#include "vox.h"

int modemfd = -1;

void
print_escaped(file, s, len)
	FILE *file;
	char *s;
	size_t len;
{
	fprintf(file, "\"");
	for (; len; s++, len--) {
		if (*s == '\n')		fprintf(file, "\\n");
		else if (*s == '\"')	fprintf(file, "\\\"");
		else if (*s == '\'')	fprintf(file, "\\'");
		else if (*s == '\r')	fprintf(file, "\\r");
		else if (*s == '\t')	fprintf(file, "\\t");
		else if (*s == '\b')	fprintf(file, "\\b");
		else if (*s == '\0')	fprintf(file, "\\0");
		else if (*s < ' ')	fprintf(file, "\\%03o", *s);
		else if (*s >= 0x7f)	fprintf(file, "\\%03o", *s);
		else			fprintf(file, "%c", *s);
	}
	fprintf(file, "\"");
}

void
print_command(file, c)
	FILE *file;
	struct command *c;
{
	fprintf(file, "%p\t", c);
	switch (c->action) {
	case CMD_ON_CLEAR:
		fprintf(file, "on clear\n");
		break;
	case CMD_ON_STR:
		fprintf(file, "on ");
		print_escaped(file, c->string, strlen(c->string));
		fprintf(file, " goto %p\n", c->dest.cmd);
		break;
	case CMD_ON_TIMEOUT:
		fprintf(file, "on timeout goto %p\n", c->dest.cmd);
		break;
	case CMD_LOCK:
		fprintf(file, "lock %s\n", c->onoff ? "on" : "off");
		break;
	case CMD_TIMER:
		fprintf(file, "timer %d\n", c->number);
		break;
	case CMD_QUOTE:
		fprintf(file, "quote ");
		print_escaped(file, c->string, strlen(c->string));
		fprintf(file, "\n");
		break;
	case CMD_WAIT:
		fprintf(file, "wait ");
		if (c->string)
			print_escaped(file, c->string, strlen(c->string));
		fprintf(file, "\n");
		break;
	case CMD_VCON:
		fprintf(file, "vcon\n");
		break;
	case CMD_DELAY:
		fprintf(file, "delay %d\n", c->number);
		break;
	case CMD_GOTO:
		fprintf(file, "goto %p\n", c->dest.cmd);
		break;
	case CMD_PLAY:
		fprintf(file, "play ");
		print_escaped(file, c->string, strlen(c->string));
		fprintf(file, "\n");
		break;
	case CMD_RECORD:
		fprintf(file, "record ");
		print_escaped(file, c->string, strlen(c->string));
		fprintf(file, "\n");
		break;
	case CMD_EXEC:
		fprintf(file, "exec ");
		print_escaped(file, c->string, strlen(c->string));
		fprintf(file, "\n");
		break;
	case CMD_SYSLOG:
		fprintf(file, "syslog ");
		print_escaped(file, c->string, strlen(c->string));
		fprintf(file, "\n");
		break;
	case CMD_SETENV:
		fprintf(file, "setenv %s ", c->envvar);
		print_escaped(file, c->string, strlen(c->string));
		fprintf(file, "\n");
		break;
	default:
		fprintf(file, "# unknown action %d\n", c->action);
	}
}

