/*	$Id: vox_y.y,v 1.4 2000/01/26 22:36:29 leonard Exp $	*/
/* Copyright 1999, David Leonard. All rights reserved. */

%{
#include <stdio.h>
#include <string.h>
#include <db.h>
#include <fcntl.h>
#include <err.h>

#include "vox.h"

char *	cat(char *, char *);
void	addlabel(char *, struct command *);

struct command * dupcmd(struct command *);
struct command * first;
%}

%union{
	struct command	cmd;
	struct command *cmdptr;
	char *		str;
	int		num;
}

%token		DLEQ LOCK GOTO WAIT PLAY EXEC CLEAR QUOTE TIMER DELAY SYSLOG
%token		VCON RECORD SETENV TIMEOUT ':' '\n'
%token		ON OFF NUMBER STRING SYMBOL
%type <cmdptr>	script	
%type <cmd>	command
%type <num>	onoff ON OFF NUMBER
%type <str>	STRING SYMBOL string

%%

top	:	script			{ first = $1; }
	;

script	:	/* empty */		{ $$ = NULL; }
	|	SYMBOL ':' script	{ addlabel($1, $3); 
					  $$ = $3; 
					  free($1); }
	|	'\n' script 		{ $$ = $2; }
	|	command '\n'		{ $1.srcname = srcname;
					  $1.srcline = srcline; }
		script			{ $$ = dupcmd(&$1);
					  $$->next = $4; }
	;

command	:	ON CLEAR		{ $$.action = CMD_ON_CLEAR; 
					  $$.string = NULL; }
	|	ON string GOTO SYMBOL	{ $$.action = CMD_ON_STR; 
					  $$.string = $2;
					  if (*($2) == '\0')
						yyerror("empty match string");
					  $$.dest.str = $4; }
	|	ON TIMEOUT GOTO SYMBOL	{ $$.action = CMD_ON_TIMEOUT;
					  $$.dest.str = $4; }
	|	LOCK onoff		{ $$.action = CMD_LOCK;
					  $$.onoff = $2; }
	|	TIMER NUMBER		{ $$.action = CMD_TIMER;
					  $$.number = $2; }
	|	TIMER CLEAR		{ $$.action = CMD_TIMER;
					  $$.number = 0; }
	|	QUOTE string		{ $$.action = CMD_QUOTE;
					  $$.string = $2; }
	|	WAIT			{ $$.action = CMD_WAIT;
					  $$.string = NULL; }
	|	WAIT string		{ $$.action = CMD_WAIT;
					  $$.string = $2; }
	|	VCON			{ $$.action = CMD_VCON; }
	|	GOTO SYMBOL		{ $$.action = CMD_GOTO;
					  $$.dest.str = $2; }
	|	DELAY NUMBER		{ $$.action = CMD_DELAY;
					  $$.number = $2; }
	|	PLAY string		{ $$.action = CMD_PLAY;
					  $$.string = $2; }
	|	RECORD string		{ $$.action = CMD_RECORD;
					  $$.string = $2; }
	|	EXEC string		{ $$.action = CMD_EXEC;
					  $$.string = $2; }
	|	SETENV SYMBOL string	{ $$.action = CMD_SETENV;
					  $$.envvar = $2;
					  $$.string = $3; }
	|	SYSLOG string		{ $$.action = CMD_SYSLOG;
					  $$.string = $2; }
	;

onoff	:	ON			{ $$ = 1; }
	|	OFF			{ $$ = 0; }
	;

string	:	STRING			{ $$ = $1; }
	|	string STRING		{ $$ = cat($1, $2); }
	|	DLEQ			{ $$ = strdup("\020"); }
	;


%%

char *
cat(a, b)
	char *a, *b;
{
	char *r = malloc(strlen(a) + strlen(b) + 1);
	strcpy(r, a);
	strcat(r, b);
	free(a);
	free(b);
	return r;
}

DB *labels = NULL;

void
initlabels()
{
	labels = dbopen(NULL, O_RDWR, 0666, DB_HASH, NULL);
	if (labels == NULL)
		err(1, "db: open hash table");
}

void
addlabel(label, cmd)
	char *label;
	struct command *cmd;
{
	DBT label_d, cmd_d;

	label_d.data = label;
	label_d.size = strlen(label);
	cmd_d.data = &cmd;
	cmd_d.size = sizeof cmd;

	if ((*labels->put)(labels, &label_d, &cmd_d, 0) == -1)
		err(1, "db: put label");
}

struct command *
dupcmd(cmd)
	struct command * cmd;
{
	struct command * newcmd;

	newcmd = (struct command *)malloc(sizeof (struct command));
	if (newcmd == NULL)
		err(1, "malloc");
	memcpy(newcmd, cmd, sizeof (struct command));
	return newcmd;
}

void
fix_dests(first)
	struct command * first;
{
	struct command * cmd;
	DBT label_d, cmd_d;

	for (cmd = first; cmd; cmd = cmd->next) {
		switch (cmd->action) {
		case CMD_ON_STR:
		case CMD_ON_TIMEOUT:
		case CMD_GOTO:
			label_d.data = cmd->dest.str;
			label_d.size = strlen(cmd->dest.str);
			cmd_d.data = NULL;
			cmd_d.size = 0;
			switch ((*labels->get)(labels, &label_d, &cmd_d, 0)) {
			case -1: err(1, "db get");
			case 1: errx(1, "%s:%d: unknown label %s",
				cmd->srcname, cmd->srcline, cmd->dest.str);
			}
			memcpy(&cmd->dest.cmd, cmd_d.data, 
			    sizeof cmd->dest.cmd);
			break;
		default:
			/* do nothing */
		}
	}
}

void
check_closure(first)
	struct command * first;
{
	struct command * cmd;

	if (first == NULL)
		errx(1, "empty script");

	/* Walk to last command */
	for (cmd = first; cmd->next != NULL; cmd = cmd->next)
		;

	/* Something that guarantees a branch is needed. */
	if (cmd->action == CMD_GOTO)
		return;
	if (cmd->action == CMD_WAIT && cmd->string == NULL)
		return;

	errx(1, "%s:%d: last line must be a 'goto' or 'wait'",
		cmd->srcname, cmd->srcline);
}

void
yyerror(msg)
	char *msg;
{
	errx(1, "%s, near line %d: %s", srcname, srcline, msg);
}
